# coding: utf-8

require 'fileutils'

require 'spec_helper'

require 'chef/knife/hitori_cook'
require 'chef/client'


describe 'hitori_cook' do
  before do
    Chef::Knife::HitoriCook.load_deps
    @obj = Chef::Knife::HitoriCook.new
  end

  describe :run do
    before do
      @conf = {
          security_groups: 'sec_group',
          availability_zone: 'avzone',
          image: 'img',
          flavor: 'flavor',
          ssh_user: 'ssh_user',
          ssh_port: 22,
          identity_file: 'id_file',
          aws_ssh_key_id: 'ssh_key_id',
          region: 'my_region',
          template_file: 'my_template',
      }
    end

    it 'should call run_remote' do
      @obj.should_receive(:run_remote)
      @obj.run
    end
  end

  describe :update_environment do
    before(:each) do
      Chef::Config[:data_bag_path_tpl] = 'databag/%s'
      Chef::Config[:settings_path_tpl] = 'settings/%s'
      Chef::Config[:encrypted_data_bag_secret_tpl] = 'enc/%s'
      @obj.update_environment('my_env')
    end

    subject {Chef::Config}
    its(:data_bag_path) {should == 'databag/my_env'}
    its(:settings_path) {should == 'settings/my_env'}
    its(:encrypted_data_bag_secret) {should == 'enc/my_env'}
  end

  #describe :json_attrs do
  #  before() do
  #    @data = {'my' => 'data', 'run_list' => 'role[magic]'}
  #    temp = Tempfile.open('hitori-')
  #    Chef::Config[:settings_path] = temp.path
  #    temp.write JSON.dump(@data)
  #    temp.close
  #  end
  #
  #  it 'should return json data of Config[:settings_path]' do
  #    @obj.json_attrs.should == @data
  #  end
  #
  #  it 'should update run_list if options is specified' do
  #    @obj.config[:recipes] = 'cat'
  #    @obj.config[:roles] = 'dog'
  #    ret = @obj.json_attrs
  #    ret['run_list'].sort.should == %w(recipe[cat] role[dog]).sort
  #  end
  #end

  describe :new_run_list do
    before(:each) do
      @obj.config[:roles] = 'role1, role2'
      @obj.config[:recipes] = 'rcp1,rcp2'
    end

    subject {@obj.new_run_list}
    it {should include('role[role1]')}
    it {should include('role[role2]')}
    it {should include('recipe[rcp1]')}
    it {should include('recipe[rcp2]')}
  end

  describe :run_remote do
    before do
      @obj.should_receive(:find_ec2_servers).and_return(%w(s1 s2))
    end

    it 'should call RunRemoteCook.run' do
      rrc = double('RunRemoteCook')
      KnifeHitori::RunRemoteCook.should_receive(:new).
          with(Chef::Config, @obj.config, 's1', anything).
          and_return(rrc)
      KnifeHitori::RunRemoteCook.should_receive(:new).
          with(Chef::Config, @obj.config, 's2', anything).
          and_return(rrc)
      rrc.should_receive(:run).twice
      @obj.run_remote
    end
  end

  describe :fetch_server_info do
    before do
      @s1, @s2, @s3 = %w(s1 s2 s3).map{|s|
        x = double(s).as_null_object
        x.stub_chain(:tags, :[])
        x.stub_chain(:tags, :[]).with('Name').and_return(s)
        x.stub_chain(:tags, :[]).with('my_tag').and_return('tag_' + s)
        x.stub(:groups => [s])
        x.stub(:public_ip_address => 'ip_'+s)
        x
      }
      @obj.should_receive(:server_info_list).and_return([@s1, @s2, @s3])
    end

    it 'should return servers by tag' do
      @obj.config[:tag] = 'my_tag=tag_s2'
      ret = @obj.fetch_server_info
      ret.size.should == 1
      ret[0].should == @s2
    end

    it 'should return servers by groups' do
      @obj.config[:groups] = 's1,s3'
      ret = @obj.fetch_server_info
      ret.size.should == 2
      ret.should == [@s1, @s3]
    end

    it 'should return servers by Name' do
      @obj.config[:name] = 's1'
      ret = @obj.fetch_server_info
      ret.size.should == 1
      ret.should == [@s1]
    end

    it 'should return servers by public_ip_address' do
      @obj.config[:public_ip_address] = 'ip_s3'
      ret = @obj.fetch_server_info
      ret.size.should == 1
      ret.should == [@s3]
    end

    it 'should return nil if no server matches' do
      ret = @obj.fetch_server_info
      ret.should == nil
    end
  end
end
